/* riprogress.c
 *
 * Send extended ripping progress information from cdparanoia to Rocco via stdout.
 *
 * 22. June 2005, Oliver Kuss, Hermstedt AG
 * (C) 2005 Hermstedt AG, All rights reserved
 */


#include "riprogress.h"

#include <unistd.h>


int gEnabled = 0;

rp_field_t gFields;
rp_num_t gCDBegin, gTrackBegin;


static void 
	write_progress (
		rp_num_t	iProgress);

#define ForAllFields( i ) \
	for (i = 0; i < erp_num_fields; ++i)


void rp_enable ()
{
	gEnabled = 1;
}


void rp_disable ()
{
	gEnabled = 0;
}


void rp_begin_rip (
	rp_num_t num_tracks, 
	rp_num_t num_sectors)
{
	int i;
	ForAllFields (i)
		gFields [i] = 0;

	gFields [erp_total_tracks] = num_tracks;
	gFields [erp_total_cd_sec] = num_sectors;
	
	gCDBegin = 0; 
	gTrackBegin = 0;
	
	write_progress (erp_begin_rip);
}


void rp_end_rip ()
{
	write_progress (erp_end_rip);
}


void rp_begin_track (const char* name, rp_num_t batch_track, rp_num_t num_sectors)
{
	uint16_t tracklen = strlen (name);

	gTrackBegin = 0;

	gFields [erp_current_track] = batch_track;
	gFields [erp_current_track_sec] = 0;
	gFields [erp_total_track_sec] = num_sectors;
	gFields [erp_promillage_track] = 0;

	write_progress (erp_begin_track);
	write (STDOUT_FILENO, &tracklen, sizeof (tracklen));
	write (STDOUT_FILENO, name, tracklen);
}


void rp_end_track (const char* name)
{
	uint16_t tracklen = strlen (name);

	gFields [erp_promillage_track] = 1000;
	write_progress (erp_end_track);

	write (STDOUT_FILENO, &tracklen, sizeof (tracklen));
	write (STDOUT_FILENO, name, tracklen);
}



void rp_end_sector (
	rp_num_t		sector, 
	rp_num_t		promille)
{
	if (gCDBegin == 0)		gCDBegin = sector;
	if (gTrackBegin == 0)	gTrackBegin = sector;

	gFields [erp_current_track_sec] = sector - gTrackBegin;
	gFields [erp_current_cd_sec] = sector - gCDBegin;
	gFields [erp_promillage_track] = promille;
	gFields [erp_promillage_cd] = gFields [erp_current_cd_sec] * 1000 / gFields [erp_total_cd_sec];

	write_progress (erp_end_sector);
}


static void 
	write_progress (
		rp_num_t	iProgress)
{
	gFields [erp_progress] = iProgress;
	static const uint16_t kProgressMarker = 0xFFFF;

	if (gEnabled)
	{
		write (STDOUT_FILENO, &kProgressMarker, sizeof (kProgressMarker));
		write (STDOUT_FILENO, gFields, sizeof (gFields));
	}
}
